@@ -45,6 +45,8 @@ static int main_help(int argc, const char *argv[]) {
     "  -q,--no-quote: turn off quote handling",
     "  -R,--skip-head <n>: skip specified number of initial rows",
     "  -d,--header-row-span <n>: apply header depth (rowspan) of n",
+    "  -u,--malformed-utf8-replacement <replacement_string>: replacement string (can be empty) in case of malformed UTF8 input",
+    "       (default for \"desc\" commamnd is '?')",
     "  -S,--keep-blank-headers: disable default behavior of ignoring leading blank rows",
     "  -v,--verbose: verbose output",
     "",
@@ -61,6 +63,7 @@ static int main_help(int argc, const char *argv[]) {
     "  2tsv     : convert to tab-delimited text",
     "  serialize: convert into 3-column format (id, column name, cell value)",
     "  stack    : stack tables vertically, aligning columns with common names",
+    "  compare  : compare two or more tables and output differences",
     "",
     "Other commands:",
     "  2db      : convert json to sqlite3 db",
