@@ -65,6 +65,7 @@ ZSV_MAIN_DECL(stack);
 ZSV_MAIN_DECL(desc);
 ZSV_MAIN_DECL(sql);
 ZSV_MAIN_DECL(2db);
+ZSV_MAIN_DECL(compare);
 ZSV_MAIN_DECL(echo);
 ZSV_MAIN_NO_OPTIONS_DECL(prop);
 ZSV_MAIN_NO_OPTIONS_DECL(rm);
@@ -95,6 +96,7 @@ struct builtin_cmd builtin_cmds[] = {
   CLI_BUILTIN_COMMAND(desc),
   CLI_BUILTIN_COMMAND(sql),
   CLI_BUILTIN_COMMAND(2db),
+  CLI_BUILTIN_COMMAND(compare),
   CLI_BUILTIN_COMMAND(echo),
   CLI_BUILTIN_NO_OPTIONS_COMMAND(prop),
   CLI_BUILTIN_NO_OPTIONS_COMMAND(rm)
