@@ -69,7 +69,7 @@ static int zsv_property_usage(FILE *target) {
 static int show_all_properties(const unsigned char *filepath) {
   int err = 0;
   if(!zsv_file_readable((const char *)filepath, &err, NULL)) {
-    perror(filepath);
+    perror((const char *)filepath);
     return err;
   }
   err = zsv_cache_print(filepath, zsv_cache_type_property, (const unsigned char *)"{}");
@@ -349,12 +349,12 @@ static int prop_arg_value(int i, int argc, const char *argv[], int64_t *value) {
     *value = ZSV_PROP_ARG_REMOVE;
   else {
     char *end = NULL;
-    intmax_t i = strtoimax(arg, &end, 0);
+    intmax_t j = strtoimax(arg, &end, 0);
     if(arg && *arg && end && *end == '\0') {
-      if(i == 0)
+      if(j == 0)
         *value = ZSV_PROP_ARG_REMOVE;
-      else if(i > 0 && i <= UINT_MAX)
-        *value = i;
+      else if(j > 0 && j <= UINT_MAX)
+        *value = j;
       return 0;
     }
     fprintf(stderr, "Invalid property value '%s'.\n", arg);
