@@ -225,7 +225,6 @@ size_t zsv_strnext_is_sign(const unsigned char *s, size_t len) {
   return 0;
 }
 
-
 /**
  * zsv_strwhite(): convert consecutive white to single space
  *
@@ -296,45 +295,6 @@ size_t zsv_strwhite(unsigned char *s, size_t len, unsigned int flags) {
   return new_len;
 }
 
-// zsv_strencode(): force UTF8 encoding
-// replace any non-conforming utf8 with the specified char, or
-// remove from the string (and shorten the string) if replace = 0.
-// returns the length of the valid string
-ZSV_EXPORT
-size_t zsv_strencode(unsigned char *s, size_t n, unsigned char replace,
-                     int (*malformed_handler)(void *, const unsigned char *s, size_t n, size_t offset), void *handler_ctx) {
-  size_t new_len = 0;
-  int clen;
-  for(size_t i2 = 0; i2 < n; i2 += (size_t)clen) {
-    clen = ZSV_UTF8_CHARLEN(s[i2]);
-    if(LIKELY(clen == 1))
-      s[new_len++] = s[i2];
-    else if(UNLIKELY(clen < 0) || UNLIKELY(i2 + clen >= n)) {
-      if(malformed_handler)
-        malformed_handler(handler_ctx, s, n, new_len);
-      if(replace)
-        s[new_len++] = replace;
-      clen = 1;
-    } else { /* might be valid multi-byte utf8; check */
-      unsigned char valid_n;
-      for(valid_n = 1; valid_n < clen; valid_n++)
-        if(!ZSV_UTF8_SUBSEQUENT_CHAR_OK(s[i2 + valid_n]))
-          break;
-      if(valid_n == clen) { /* valid_n utf8; copy it */
-        memmove(s + new_len, s + i2, clen);
-        new_len += clen;
-      } else { /* invalid; valid_n smaller than expected */
-        if(malformed_handler)
-          malformed_handler(handler_ctx, s, n, new_len);
-        memset(s + new_len, replace, valid_n);
-        new_len += valid_n;
-        clen = valid_n;
-      }
-    }
-  }
-  return new_len; // new length
-}
-
 size_t zsv_strip_trailing_zeros(const char *s, size_t len) {
   if(len && memchr(s, '.', len)) {
     while(len && s[len-1] == '0')
@@ -344,3 +304,30 @@ size_t zsv_strip_trailing_zeros(const char *s, size_t len) {
   }
   return len;
 }
+
+/**
+ * zsv_strunescape_backslash(): convert consecutive white to single space
+ *
+ * @param s     string to convert
+ * @param len   length of input string
+ * @param flags bitfield of ZSV_STRWHITE_FLAG_XXX values
+ */
+size_t zsv_strunescape_backslash(unsigned char *s, size_t len) {
+  if(len == 0 || !memchr(s, '\\', len - 1))
+    return len;
+  size_t j = 0;
+  for(size_t i = 0; i < len; i++, j++) {
+    if(UNLIKELY(s[i] == '\\' && i + 1 < len && memchr("tnr", s[i+1], 3))) {
+      ++i;
+      s[j] = s[i] == 't' ? '\t' : s[i] == 'n' ? '\n' : '\r';
+    } else
+      s[j] = s[i];
+  }
+  return j;
+}
+
+struct zsv_cell zsv_get_cell_trimmed(zsv_parser parser, size_t ix) {
+  struct zsv_cell c = zsv_get_cell(parser, ix);
+  c.str = (unsigned char *)zsv_strtrim(c.str, &c.len);
+  return c;
+}
