@@ -21,9 +21,15 @@
 #define ARRAY_SIZE(array) (sizeof(array) / sizeof(array[0]))
 #endif
 
-void to_unicode(const void *path, wchar_t *wbuf, size_t wbuf_len);
+void zsv_win_to_unicode(const void *path, wchar_t *wbuf, size_t wbuf_len);
 
 int zsv_replace_file(const void *src, const void *dest);
+
+/**
+ * Windows does not have perror(), so we define our own printLastError()
+ */
+void zsv_win_printLastError();
+
 #endif
 
 #endif
