@@ -28,6 +28,46 @@ const char *zsv_lib_version(void) {
   return ZSV_VERSION;
 }
 
+/**
+ * Ensure valid UTF8 encoding by, if needed, replacing malformed bytes
+ */
+ZSV_EXPORT
+size_t zsv_strencode(unsigned char *s, size_t n, unsigned char replace,
+                     int (*malformed_handler)(void *, const unsigned char *s, size_t n, size_t offset), void *handler_ctx) {
+  size_t new_len = 0;
+  int clen;
+  for(size_t i2 = 0; i2 < n; i2 += (size_t)clen) {
+    clen = ZSV_UTF8_CHARLEN(s[i2]);
+    if(LIKELY(clen == 1))
+      s[new_len++] = s[i2];
+    else if(UNLIKELY(clen < 0) || UNLIKELY(i2 + clen >= n)) {
+      if(malformed_handler)
+        malformed_handler(handler_ctx, s, n, new_len);
+      if(replace)
+        s[new_len++] = replace;
+      clen = 1;
+    } else { /* might be valid multi-byte utf8; check */
+      unsigned char valid_n;
+      for(valid_n = 1; valid_n < clen; valid_n++)
+        if(!ZSV_UTF8_SUBSEQUENT_CHAR_OK(s[i2 + valid_n]))
+          break;
+      if(valid_n == clen) { /* valid_n utf8; copy it */
+        memmove(s + new_len, s + i2, clen);
+        new_len += clen;
+      } else { /* invalid; valid_n smaller than expected */
+        if(malformed_handler)
+          malformed_handler(handler_ctx, s, n, new_len);
+        if(replace) {
+          memset(s + new_len, replace, valid_n);
+          new_len += valid_n;
+        }
+        clen = valid_n;
+      }
+    }
+  }
+  return new_len; // new length
+}
+
 /**
  * When we parse a chunk, if it was not the first parse call, we might have a partial
  * row at the end of our buffer that must be moved. The reason we do this at the beginning
@@ -215,6 +255,8 @@ char zsv_quoted(zsv_parser parser) {
   return parser->quoted || parser->opts.no_quotes;
 }
 
+// to do: benchmark returning zsv_cell struct vs just a zsv_cell pointer
+ZSV_EXPORT
 struct zsv_cell zsv_get_cell(zsv_parser parser, size_t ix) {
   if(ix < parser->row.used)
     return parser->row.cells[ix];
@@ -344,7 +386,7 @@ enum zsv_status zsv_finish(struct zsv_scanner *scanner) {
   if(!scanner->finished) {
     scanner->finished = 1;
     if(!scanner->abort) {
-      if(scanner->scanned_length > scanner->cell_start)
+      if(scanner->scanned_length > 0 && scanner->scanned_length >= scanner->cell_start)
         cell_dl(scanner, scanner->buff.buff + scanner->cell_start,
                 scanner->scanned_length - scanner->cell_start);
       if(scanner->have_cell) {
